//**************************************************************************/
// Copyright (c) 1998-2007 Autodesk, Inc.
// All rights reserved.
// 
//  Use of this software is subject to the terms of the Autodesk license 
//  agreement provided at the time of installation or download, or which 
//  otherwise accompanies this software in either electronic or hard copy form.
//**************************************************************************/
// DESCRIPTION: Interface for Biped for Max 11
// AUTHOR: Susan Amkraut - created August 31, 2007
//***************************************************************************/

#pragma once

#include "biped10Api.h"


/*! \defgroup BipedInterface11 Biped Interface 11
Max 11 Biped export interface
*/

//! \brief Interface into the biped driver controller for Max 11.
/*! This contains functions supporting new biped features in Max 11.
*/
class IBipDriver11 : public IBipDriver10
{
	public:

	/************ Access to the Biped Interface ***************/
	/*! \name Biped Interface Access
	*/
//!@{
		//! \deprecated This has been deprecated as of 3ds Max 2022, please use I_BIPDRIVER11 instead [insensitive language]
		MAX_DEPRECATED static const ULONG I_BIPMASTER11 = 0x9171;    //!< Biped interface identifier
		static constexpr ULONG I_BIPDRIVER11 = 0x9171;    //!< Biped interface identifier

		//! \deprecated This has been deprecated as of 3ds Max 2022, please use GetBipDriver11Interface instead [insensitive language]
		MAX_DEPRECATED BIPExport static IBipDriver11* GetBipMaster11Interface(Animatable& anim);
		//! \brief Gets the Biped interface
		//! \param[in]  anim - the Animatable whose Biped interface is returned
		//! \return the Biped interface
		BIPExport static IBipDriver11* GetBipDriver11Interface(Animatable& anim); //!< Gets the Biped interface
//!@}*/

	/************ Triangle Neck ***************/
	/*! \name Triangle Neck
	If Triangle Neck is true, the clavicles are parented to the top spine link.
	If Triangle Neck is false, the clavicles are parented to the neck base.
	*/
//!@{		
		//! \brief Queries the value of Triangle Neck
		//! \return true if Triangular Neck is on, false if not
		virtual const BOOL	GetTriangleNeck()=0;

		//! \brief Sets Triangle Neck
		//! \param[in]  bEnabled - sets Triangle Neck to the value of bEnabled
		virtual void	SetTriangleNeck(BOOL bEnabled)=0;
//!@}
		
	/************ ForeFeet ***************/
	/*! \name ForeFeet
	If ForeFeet is true, the the fingers are shaped and attached like toes, and viewport manipulation of IK hands is like feet.
	If ForeFeet is false, the fingers and hands appear and behave as usual.
	*/
//!@{		
		//! \brief Queries the value of ForeFeet
		//! \return true if ForeFeet is on, false if not
		virtual const BOOL	GetForeFeet()=0;

		//! \brief Sets ForeFeet
		//! \pre must be in figure mode
		//! \param[in]  bEnabled - sets ForeFeet to the value of bEnabled
		virtual void	SetForeFeet(BOOL bEnabled)=0;
//!@}
		
	/************ Mirror In Place ***************/
	/*! \name Mirror In Place
	*/
//!@{		
		//! \brief Mirrors the biped animation in place
		/*! The position and orientation of the center of mass at the current frame is maintained.
		\pre cannot be in figure mode, motion flow mode, or mixer mode
		\pre cannot be editing a biped layer
		\return true if the operation was successful, false if the operation was unsuccessful
		*/
		virtual bool	MirrorInPlace()=0;
//!@}
};

//! \deprecated This has been deprecated as of 3ds Max 2022, please use IBipDriver11 instead [insensitive language]
typedef MAX_DEPRECATED IBipDriver11 IBipMaster11;
