//**************************************************************************/
// Copyright 2020 Autodesk, Inc.  All rights reserved.
//
// Use of this software is subject to the terms of the Autodesk license agreement
// provided at the time of installation or download, or which otherwise accompanies 
// this software in either electronic or hard copy form.   

#pragma once

#include "iFnPub.h"
#include "GetCOREInterface.h"

#define AutoBackup_INTERFACE Interface_ID(0x4dab68f9, 0x3e4032c6)

//! \brief Interface class for the auto save manager.
/*! Interface for the Auto backup manager class. This class has only one instance
	and is responsible for automatically saving the max scene at periodic time
	intervals. 
*/
class IAutoBackup : public FPStaticInterface
{
	public:
		//! Turns on and off Auto Save.
		virtual void SetEnable(bool b)		= 0;
		virtual bool  GetEnable() const		= 0;
		//! Sets and Gets the number of auto save files.
		virtual void  SetNumFiles(int n)	= 0;
		virtual int   GetNumFiles() const 	= 0;
		//! Sets and Gets the time between saves, in minutes.
		virtual void  SetInterval(float i)	= 0;
		virtual float GetInterval() const 	= 0;
		//! Tells whether an auto save operation is currently active.
		virtual bool  GetActive() const		= 0;
		//! Gets and Sets the filename portion of the auto save file.
		/*! This does not include the path, or the filename extension suffix.*/
		virtual TSTR  GetFileName() const	= 0;
		virtual void  SetFileName(const TCHAR *fn) = 0;
		/*! Turns on and off Bailout during Auto Save.
		Allows user to abort save by pressing escape key
		\sa  Class MaxSDK::Util::IBailOutBusyProcessManager
		*/
		virtual void  SetBailoutEnable(bool b) = 0;
		virtual bool  GetBailoutEnable() const = 0;
};
//! \brief Gets the IAutobackup interface pointer.
inline IAutoBackup* GetAutoBackupInterface()
{
	return static_cast<IAutoBackup*>(GetCOREInterface(AutoBackup_INTERFACE));
}

