/**********************************************************************
 *<
	FILE:  ikctrl.h

	DESCRIPTION:  Inverse Kinematics Controllers

	CREATED BY:  Rolf Berteig

	HISTORY: 3-1-97

 *>	Copyright (c) 1997, All Rights Reserved.
 **********************************************************************/
#pragma once

#include "ref.h"
#include "control.h"

#define IKDRIVER_CLASSID Class_ID(0xa91004be,0x9901fe83)
 //! \deprecated This has been deprecated as of 3ds Max 2022, please use IKDRIVER_CLASSID instead [insensitive language]
#define IKMASTER_CLASSID Class_ID(0xa91004be,0x9901fe83)
#pragma deprecated("IKMASTER_CLASSID")

#define IKDRIVEN_CLASSID Class_ID(0xbe380a31,0x310dc9e4)
//! \deprecated This has been deprecated as of 3ds Max 2022, please use IKDRIVEN_CLASSID instead [insensitive language]
#define IKSLAVE_CLASSID Class_ID(0xbe380a31,0x310dc9e4)
#pragma deprecated("IKSLAVE_CLASSID")

/*! \sa  Class ReferenceTarget, Class IKDrivenControl, Class Control.\n\n
\par Description:
The IK Controller requires that you use the Bones system. When you create the
bones, a driven IK controller is assigned to each bone. All of the driven IK
controllers in a single hierarchy are, in turn, controlled by a driver IK
controller. This class provides two methods to work with the driver controller.
To get an interface to this class call <b>GetInterface(I_DRIVER);</b>on the
controller in question. If the return value is non-NULL you can cast the
pointer to an instance of this class.\n\n
For an example the use of this class see
<b>/MAXSDK/SAMPLES/OBJECTS/BONES.CPP</b>.  */
class IKDriverControl : public ReferenceTarget {
	public:
		Class_ID ClassID() override { return IKDRIVER_CLASSID; }
		SClass_ID SuperClassID() override { return REF_TARGET_CLASS_ID; }

		/*! \remarks Adds the specified node to the list of driven nodes maintained.
		\par Parameters:
		<b>INode *node</b>\n\n
		The node to add. */
		virtual void AddDrivenNode(INode* node) = 0;
		//! \deprecated This has been deprecated as of 3ds Max 2022, please use AddDrivenNode instead [insensitive language]
		MAX_DEPRECATED virtual void AddSlaveNode(INode* node) final { AddDrivenNode(node); }
		/*! \remarks Returns a pointer to the IK driver object. */
		virtual void* GetIKDriverBase() = 0;
		//! \deprecated This has been deprecated as of 3ds Max 2022, please use GetIKDriverBase instead [insensitive language]
		MAX_DEPRECATED virtual void* GetMasterBase() final { return GetIKDriverBase(); }

		/*! \remarks This method is available in release 3.0 and later
		only.\n\n
		Sets the position threshold. If the UI for the driver controller is not
		visible while this method is called, the screen is not redrawn to
		reflect the changes. After calling this method you should therefore
		call <b>Interface::RedrawViews()</b>.
		\par Parameters:
		<b>float t</b>\n\n
		The value to set. */
		virtual void		SetPosThresh(float t)=0;
		/*! \remarks This method is available in release 3.0 and later
		only.\n\n
		Sets the rotation threshold. If the UI for the driver controller is not
		visible while this method is called, the screen is not redrawn to
		reflect the changes. After calling this method you should therefore
		call <b>Interface::RedrawViews()</b>.
		\par Parameters:
		<b>float t</b>\n\n
		The value to set. */
		virtual void		SetRotThresh(float t)=0;
		/*! \remarks This method is available in release 3.0 and later
		only.\n\n
		Sets the iterations value. If the UI for the driver controller is not
		visible while this method is called, the screen is not redrawn to
		reflect the changes. After calling this method you should therefore
		call <b>Interface::RedrawViews()</b>.
		\par Parameters:
		<b>int i</b>\n\n
		The value to set. */
		virtual void		SetIterations(int i)=0;
		/*! \remarks This method is available in release 3.0 and later
		only.\n\n
		Sets the start time. If the UI for the driver controller is not visible
		while this method is called, the screen is not redrawn to reflect the
		changes. After calling this method you should therefore call
		<b>Interface::RedrawViews()</b>.
		\par Parameters:
		<b>TimeValue s</b>\n\n
		The time to set. */
		virtual void		SetStartTime(TimeValue s)=0;
		/*! \remarks This method is available in release 3.0 and later
		only.\n\n
		Sets the end time. If the UI for the driver controller is not visible
		while this method is called, the screen is not redrawn to reflect the
		changes. After calling this method you should therefore call
		<b>Interface::RedrawViews()</b>.
		\par Parameters:
		<b>TimeValue e</b>\n\n
		The time to set. */
		virtual void		SetEndTime(TimeValue e)=0;

		/*! \remarks This method is available in release 3.0 and later
		only.\n\n
		Returns the position threshold. */
		virtual float		GetPosThresh()=0;
		/*! \remarks This method is available in release 3.0 and later
		only.\n\n
		Returns the rotation threshold. */
		virtual float		GetRotThresh()=0;
		/*! \remarks This method is available in release 3.0 and later
		only.\n\n
		Returns the iterations setting. */
		virtual int			GetIterations()=0;
		/*! \remarks This method is available in release 3.0 and later
		only.\n\n
		Returns the start time. */
		virtual TimeValue	GetStartTime()=0;
		/*! \remarks This method is available in release 3.0 and later
		only.\n\n
		Returns the end time. */
		virtual TimeValue	GetEndTime()=0;

		virtual void RemoveIKChainControllers(TimeValue t)=0;
};
//! \deprecated This has been deprecated as of 3ds Max 2022, please use IKDriverControl instead [insensitive language]
typedef MAX_DEPRECATED IKDriverControl IKMasterControl;

/*! \sa  Class Control, Class IKDriverControl, Class Point3,  Class Quat.\n\n
\par Description:
The IK Controller requires that you use the Bones system. When you create the
bones, a driven IK controller is assigned to each bone. All of the driven IK
controllers in a single hierarchy are, in turn, controlled by a driver IK
controller. This class provides access to the driven controller parameters.\n\n
If you have a pointer to a TM controller you may test the Class_ID against
<b>IKDRIVEN_CLASSID</b> to determine if it is an IK Driven Controller.\n\n
For an example the use of this class see
<b>/MAXSDK/SAMPLES/OBJECTS/BONES.CPP</b>.  */
class IKDrivenControl : public Control {
	public:
		Class_ID ClassID() override { return IKDRIVEN_CLASSID; }
		SClass_ID SuperClassID() override { return CTRL_MATRIX3_CLASS_ID; }

		/*! \remarks Returns a pointer to the <b>IKDriverControl</b> that
		manages this IK driven controller. */
		virtual IKDriverControl* GetIKDriver() = 0;
		//! \deprecated This has been deprecated as of 3ds Max 2022, please use GetIKDriver instead [insensitive language]
		MAX_DEPRECATED virtual IKDriverControl* GetMaster() final { return GetIKDriver(); }
		/*! \remarks Sets the specified degree of freedom to on or off.
		\par Parameters:
		<b>int which</b>\n\n
		Specifies which degree of freedom to modify. Pass a value between 1 and
		6. The first 3 DOFs are position and the next 3 are rotation.\n\n
		<b>BOOL onOff</b>\n\n
		TRUE for on; FALSE for off. */
		virtual void SetDOF(int which,BOOL onOff)=0;
		/*! \remarks Sets the initial position to the value passed.
		\par Parameters:
		<b>Point3 pos</b>\n\n
		The initial position to set. */
		virtual void SetInitPos(Point3 pos)=0;
		/*! \remarks Sets the initial rotation to the value passed.
		\par Parameters:
		<b>Point3 rot</b>\n\n
		The initial rotation to set. */
		virtual void SetInitRot(Point3 rot)=0;
		/*! \remarks For any IK solution you explicitly move one object. 3ds
		Max uses IK calculations to move and rotate all other objects in the
		kinematic chain to react to the object you moved. The object that you
		move is the end effector. There are two types of end effectors;
		Position and Rotation. This method create or deletes an end effector
		for position or rotation or both.
		\par Parameters:
		<b>BOOL onOff</b>\n\n
		Controls if the end effector is on or off (creates or deletes the end
		effector controller). TRUE to create; FALSE to delete.\n\n
		<b>DWORD which</b>\n\n
		Specifies which end effector(s) to modify. Set the low order bit for
		position, the second bit for rotation, or set both for position and
		rotation.\n\n
		<b>Point3 pos</b>\n\n
		The initial position set at time 0 for the position controller
		(CTRL_ABSOLUTE).\n\n
		<b>Quat rot</b>\n\n
		The initial rotation set at time 0 for the rotation controller
		(CTRL_ABSOLUTE). */
		virtual void MakeEE(BOOL onOff,DWORD which,Point3 pos,Quat rot)=0;
};
//! \deprecated This has been deprecated as of 3ds Max 2022, please use IKDrivenControl instead [insensitive language]
typedef MAX_DEPRECATED IKDrivenControl IKSlaveControl;

/*! \remarks This global function creates a new IK driver controller. */
CoreExport IKDriverControl* CreateIKDriverControl();
//! \deprecated This has been deprecated as of 3ds Max 2022, please use CreateIKDriverControl instead [insensitive language]
MAX_DEPRECATED CoreExport IKDriverControl* CreateIKMasterControl();

/*! \remarks This global function creates and returns a pointer to a new IK
driven controller by specifying the driver controller and a node to which the
driven controller is assigned.
\par Parameters:
<b>IKDriverControl* driver</b>\n\n
Points to the driver controller to use.\n\n
<b>INode* drivenNode</b>\n\n
Points to the driven node to use. */
CoreExport IKDrivenControl* CreateIKDrivenControl(IKDriverControl* driver, INode* drivenNode);
//! \deprecated This has been deprecated as of 3ds Max 2022, please use CreateIKDrivenControl instead [insensitive language]
MAX_DEPRECATED CoreExport IKDrivenControl* CreateIKSlaveControl(IKDriverControl*, INode*);
