//**************************************************************************/
// Copyright 2020 Autodesk, Inc.  All rights reserved.
//
// Use of this software is subject to the terms of the Autodesk license agreement
// provided at the time of installation or download, or which otherwise accompanies 
// this software in either electronic or hard copy form.  

#pragma once

/*! \defgroup ScriptSourceTypes Script Source Types
Script Source types are used when compiling MAXScript code and specifies whether the source of the 
MAXScript code was a scene file (i.e., a scene file embedded script) or not. 
The Script Source type is used when executing the MAXScript code to determine the security rights
of the code. For example, based on security settings a scene file embedded script may not be able to create
most dotnet classes.
\n\n
*/

namespace MAXScript {
	enum class ScriptSource : int
	{
		NotSpecified, 	//!< source of the script was not specified; same security rights as for embedded scripts will apply
		Embedded,		//!< source of the script was a scene file (i.e., a scene file embedded script)
		NonEmbedded		//!< source of the script was not a scene file.
	};
}