#pragma once

//**************************************************************************/
// Copyright (c) 1998-2020 Autodesk, Inc.
// All rights reserved.
//
//  Use of this software is subject to the terms of the Autodesk license 
//  agreement provided at the time of installation or download, or which 
//  otherwise accompanies this software in either electronic or hard copy form.  
//**************************************************************************/
// DESCRIPTION: USD IO callback definition.
// AUTHOR: Autodesk Inc.
//***************************************************************************/

#include "MaxUsd/MaxUSDAPI.h"

#include <maxscript/foundation/name.h>
#include <maxscript/foundation/ValueHolderMember.h>

#include "MaxUsd.h"

namespace MAXUSD_NS_DEF {

enum class USDIOEventType
{
	None,
	OnCompletion,
};

/**
* \brief Callback related methods
**/
class USDIOCallback
{
public:
	/**
	 * \brief Constructor
	 * \param callbackFunction The maxscript/python callback, wrapped in a value holder to prevent
	 * garbage collection.
	 */
	MaxUSDAPI USDIOCallback(std::shared_ptr<ValueHolderMember> callbackFunction);
	
	/**
	 * \brief Get the callback as a Value type.
	 * \return The callback.
	 */
	MaxUSDAPI Value* GetCallback() const;
	
	/**
	 * \brief Executes the callback.
	 * \param args The arguments to pass to the callback function.
	 * \param argCount Argument count.
	 */
	MaxUSDAPI void Execute(Value** args, int argCount);

protected:
	std::shared_ptr<ValueHolderMember> callbackFunction;
};
}