#pragma once
//**************************************************************************/
// Copyright (c) 2022 Autodesk, Inc.
// All rights reserved.
//
//  Use of this software is subject to the terms of the Autodesk license 
//  agreement provided at the time of installation or download, or which 
//  otherwise accompanies this software in either electronic or hard copy form.  
//**************************************************************************/
#include <MaxUsd/MaxUSDAPI.h>

#include <pxr/base/tf/declarePtrs.h>
#include <pxr/base/tf/refPtr.h>
#include <pxr/pxr.h>

PXR_NAMESPACE_OPEN_SCOPE

TF_DECLARE_REF_PTRS(MaxUsdExportChaser);

/// \brief base class for plugin chasers which are plugins that run after the
/// core usdExport out of 3ds Max.
///
/// Chaser objects will be constructed after the initial export.
/// Chasers should save necessary data when constructed. The constructor receives
/// the Context. Save what you need from it so that you can make use of the information
/// at the Chaser execution later.
///
/// Chasers should not modify the structure of the USD file. Use this to make
/// small changes or to add attributes, in a non-destructive way, to an exported stage.
class MaxUsdExportChaser : public TfRefBase
{
public:
    ~MaxUsdExportChaser() override { }

    /// Do custom post-processing that needs to run after the main MaxUsd
    /// export loop.
    /// At this point, all data has been authored to the stage (except for
    /// any custom data that you'll author in this step).
	virtual bool MaxUSDAPI PostExport();
};

PXR_NAMESPACE_CLOSE_SCOPE
