#pragma once
//**************************************************************************/
// Copyright (c) 2022 Autodesk, Inc.
// All rights reserved.
//
// Use of this software is subject to the terms of the Autodesk license 
// agreement provided at the time of installation or download, or which 
// otherwise accompanies this software in either electronic or hard copy form.  
//**************************************************************************/

#include "MaxUsd/MaxUSDAPI.h"

#include <MaxUsd/Translators/ShadingModeExporterContext.h>
#include <MaxUsd/Translators/WriteJobContext.h>

#include <pxr/pxr.h>
#include <pxr/usd/sdf/path.h>
#include <pxr/usd/usdShade/material.h>

#include <functional>
#include <memory>

PXR_NAMESPACE_OPEN_SCOPE

class MaxUsdShadingModeExporter
{
public:
	MaxUSDAPI
	MaxUsdShadingModeExporter();
	MaxUSDAPI
	virtual ~MaxUsdShadingModeExporter();

	MaxUSDAPI
	void DoExport(
		MaxUsdWriteJobContext&               writeJobContexts,
		const pxr::TfHashSet<pxr::SdfPath, pxr::SdfPath::Hash>& primsToMaterialBind);

	/// Called once, before any exports are started.
	MaxUSDAPI
	virtual void PreExport(MaxUsdShadingModeExportContext* /* context */) {};

	/// Called inside of a loop, per-material
	MaxUSDAPI
	virtual void Export(
		const MaxUsdShadingModeExportContext& context,
		UsdShadeMaterial* const               mat,
		SdfPathSet* const                     boundPrimPaths)
		= 0;

	/// Called once, after Export is called for all shading engines.
	MaxUSDAPI
	virtual void PostExport(const MaxUsdShadingModeExportContext& /* context */) {};
};

using MaxUsdShadingModeExporterPtr = std::shared_ptr<MaxUsdShadingModeExporter>;
using MaxUsdShadingModeExporterCreator
	= std::function<std::shared_ptr<MaxUsdShadingModeExporter>()>;

PXR_NAMESPACE_CLOSE_SCOPE
