#pragma once
//**************************************************************************/
// Copyright (c) 2022 Autodesk, Inc.
// All rights reserved.
//
// Use of this software is subject to the terms of the Autodesk license 
// agreement provided at the time of installation or download, or which 
// otherwise accompanies this software in either electronic or hard copy form.  
//**************************************************************************/

#include <MaxUsd/MaxUSDAPI.h>
#include <MaxUsd/Translators/WriteJobContext.h>

#include <pxr/base/tf/token.h>
#include <pxr/base/vt/types.h>
#include <pxr/pxr.h>
#include <pxr/usd/sdf/path.h>
#include <pxr/usd/usd/prim.h>
#include <pxr/usd/usd/stage.h>

#include <string>
#include <utility>
#include <vector>

class Mtl;

PXR_NAMESPACE_OPEN_SCOPE

#define MAT_ROOT_TOKENS ((materials, "Materials"))
TF_DECLARE_PUBLIC_TOKENS(MaterialsRootMaxTokens, MaxUSDAPI, MAT_ROOT_TOKENS);

class MaxUsdShadingModeExportContext
{
public:
	MaxUSDAPI MaxUsdShadingModeExportContext(
		MaxUsdWriteJobContext&               writeJobContext);

	void SetMaterialAndBindings(Mtl* material, const std::list<SdfPath>* bindings)
	{
		this->material = material;
		this->bindings = bindings;
	}
	Mtl* GetMaterial() const { return material; }
	const std::list<SdfPath>& GetBindings() const { return *bindings; }

	const UsdStageRefPtr& GetUsdStage() const { return stage; }

	MaxUsdWriteJobContext& GetWriteJobContext() const { return writeJobContext; }

	const USDSceneBuilderOptions& GetExportArgs() const { return writeJobContext.GetArgs(); }

	/// Use this function to create a UsdShadeMaterial prim at a "standard"
	/// location computed by browsing the \p assignmentsToBind
	MaxUSDAPI UsdPrim MakeStandardMaterialPrim() const;

	/// Use this function to bind a UsdShadeMaterial prim to known bindings.
	MaxUSDAPI void BindStandardMaterialPrim(const UsdPrim& materialPrim) const;

private:
	/// The 3ds Max material element to process only in the Export calls (not set in Pre/Post Export)
	Mtl* material;
	/// The USD prim bindings for the export material element in the Export calls (not set in Pre/Post Export)
	const std::list<SdfPath>* bindings;

	/// The stage to which everything is being exported to
	const UsdStageRefPtr&                    stage;
	/// The job context for shared data required by the MaxUsdShaderWriters
	MaxUsdWriteJobContext&                   writeJobContext;
};

PXR_NAMESPACE_CLOSE_SCOPE
