#pragma once
//**************************************************************************/
// Copyright (c) 2022 Autodesk, Inc.
// All rights reserved.
//
// Use of this software is subject to the terms of the Autodesk license 
// agreement provided at the time of installation or download, or which 
// otherwise accompanies this software in either electronic or hard copy form.  
//**************************************************************************/

#include "MaxUsd/Builders/USDSceneBuilderOptions.h"

PXR_NAMESPACE_OPEN_SCOPE

class MaxUsd_MaterialWriteJob;

/// Provides basic functionality and access to shared data for prim and shader writers.
class MaxUsdWriteJobContext
{
public:
	/**
	 * \brief Constructor.
	 */
	MaxUSDAPI MaxUsdWriteJobContext(
		UsdStageRefPtr stage,
		const std::string& filename,
		const USDSceneBuilderOptions& args);

	/**
	 * \brief Destructor.
	 */
	MaxUSDAPI ~MaxUsdWriteJobContext() = default;

	/**
	 * \brief Returns the export arguments.
	 * \return Export arguments
	 */
	const USDSceneBuilderOptions& GetArgs() const { return args; }

	/**
	 * \brief The USD Stage we are inthe process of building.
	 * \return The usd
	 */
	const UsdStageRefPtr& GetUsdStage() const { return stage; }

	/**
	 * \brief Gets the file we are exporting to.
	 * \return The file path.
	 */
	const std::string& GetFilename() const { return filename; }

	/**
	 * \brief Sets the current nodes being exported and their respective paths
	 * \param nodeToPrim map with the nodes being exported and their paths 
	 */
	void SetNodesToPrimsMap(const std::map<INode*, SdfPath>& nodesToPrims);

	/**
	 * \brief Returns the nodes being exported and their respective prim paths.
	 * \return Map with the nodes and their prim path.
	 */
	const std::map<INode*, SdfPath>& GetNodesToPrimsMap() const { return maxNodesToPrims; }

protected:
	// Args for the export (any & all export options).
	const USDSceneBuilderOptions& args;

	// Stage used to write out USD file
	UsdStageRefPtr stage;
	// Filename for the USD file
	std::string filename;
	// Max nodes being exported and their prims
	std::map<INode*, SdfPath> maxNodesToPrims;
};

PXR_NAMESPACE_CLOSE_SCOPE
