//**************************************************************************/
// Copyright (c) 2022 Autodesk, Inc.
// All rights reserved.
//
// These coded instructions, statements, and computer programs contain
// unpublished proprietary information written by Autodesk, Inc., and are
// protected by Federal copyright law. They may not be disclosed to third
// parties or copied or duplicated in any form, in whole or in part, without
// the prior written consent of Autodesk, Inc.
//**************************************************************************/
#pragma once

#include "MaxUsd.h"
#include <ipoint2.h>
#include <point3.h>
#include <matrix3.h>
#include <Graphics/Matrix44.h>

#include <pxr/base/gf/vec2i.h>
#include <pxr/base/gf/vec3d.h>
#include <pxr/base/gf/vec4f.h>
#include <pxr/base/gf/matrix4d.h>


namespace MAXUSD_NS_DEF {

inline pxr::GfVec2i toUSD(IPoint2 p)
{
	return pxr::GfVec2i(p.x, p.y);
}

inline pxr::GfVec3d toUSD(Point3 p)
{
	return pxr::GfVec3d(p.x, p.y, p.z);
}

inline pxr::GfMatrix4d toUSD(Matrix3 m)
{
	pxr::GfMatrix4d r;

	for (int i = 0; i < 4; ++i)
	{
		r.SetRow3(i, toUSD(m.GetRow(i)));
	}

	r.SetColumn(3, pxr::GfVec4d(0, 0, 0, 1));
	return r;
}

inline pxr::GfMatrix4d toUSD(MaxSDK::Graphics::Matrix44 m)
{
	pxr::GfMatrix4d r;
	for (int i = 0; i < 4; ++i)
	{
		r.SetRow(i, pxr::GfVec4d(pxr::GfVec4f(m.m[i])));
	}

	return r;
}

inline Point3 toMax(pxr::GfVec3d p)
{
	return Point3(p[0], p[1], p[2]);
}

inline MaxSDK::Graphics::Matrix44 toMax(const pxr::GfMatrix4d& m)
{
	MaxSDK::Graphics::Matrix44 r;
	std::copy_n(m.data(), 16, &r.m[0][0]);

	return r;
}

inline Matrix3 toMaxMatrix3(const MaxSDK::Graphics::Matrix44& mat)
{
	Matrix3 matrix3;
	const auto row1 = mat.m[0];
	matrix3.SetRow(0, Point3(row1[0], row1[1], row1[2]));
	const auto row2 =  mat.m[1];
	matrix3.SetRow(1, Point3(row2[0], row2[1], row2[2]));
	const auto row3 =  mat.m[2];
	matrix3.SetRow(2, Point3(row3[0], row3[1], row3[2]));
	const auto row4 =  mat.m[3];
	matrix3.SetRow(3, Point3(row4[0], row4[1], row4[2]));
	return matrix3;
}

inline Matrix3 toMaxMatrix3(const pxr::GfMatrix4d& mat)
{
	Matrix3 matrix3;
	const auto row1 = mat.GetRow3(0).data();
	matrix3.SetRow(0, Point3(row1[0], row1[1], row1[2]));
	const auto row2 = mat.GetRow3(1).data();
	matrix3.SetRow(1, Point3(row2[0], row2[1], row2[2]));
	const auto row3 = mat.GetRow3(2).data();
	matrix3.SetRow(2, Point3(row3[0], row3[1], row3[2]));
	const auto row4 = mat.GetRow3(3).data();
	matrix3.SetRow(3, Point3(row4[0], row4[1], row4[2]));
	return matrix3;
}

} // namespace USDCore